import React, { useState, useEffect } from 'react';
import Link from 'next/link';
import { useRouter } from 'next/router';
import { FaEdit, FaTrash, FaUserPlus, FaSearch, FaFilter, FaSort } from 'react-icons/fa';
import { get, del } from '@/lib/api-client';
import Button from '@/components/common/Button';
import Input from '@/components/common/Input';
import Pagination from '@/components/common/Pagination';
import Modal from '@/components/common/Modal';
import { User } from '@/types/models';
import styles from '@/styles/admin/UserTable.module.scss';

interface UserTableProps {
  initialUsers?: User[];
  initialTotal?: number;
}

const UserTable: React.FC<UserTableProps> = ({ initialUsers = [], initialTotal = 0 }) => {
  const router = useRouter();
  const [users, setUsers] = useState<User[]>(initialUsers);
  const [loading, setLoading] = useState<boolean>(false);
  const [searchTerm, setSearchTerm] = useState<string>('');
  const [page, setPage] = useState<number>(1);
  const [totalUsers, setTotalUsers] = useState<number>(initialTotal);
  const [sort, setSort] = useState<string>('createdAt_desc');
  const [filter, setFilter] = useState<string>('all');
  const [deleteConfirmOpen, setDeleteConfirmOpen] = useState<boolean>(false);
  const [selectedUser, setSelectedUser] = useState<User | null>(null);
  
  const limit = 10;
  
  useEffect(() => {
    // Only fetch if we don't have initial data or if params change
    if (!initialUsers.length || router.query.page || router.query.search || router.query.sort) {
      fetchUsers();
    }
  }, [page, searchTerm, sort, filter]);
  
  const fetchUsers = async () => {
    try {
      setLoading(true);
      
      const queryParams = new URLSearchParams({
        page: page.toString(),
        limit: limit.toString(),
        sort,
        filter,
      });
      
      if (searchTerm) {
        queryParams.append('search', searchTerm);
      }
      
      const response = await get<{ users: User[], total: number }>(
        `/api/admin/users?${queryParams.toString()}`
      );
      
      if (response.success && response.data) {
        setUsers(response.data.users);
        setTotalUsers(response.data.total);
      }
    } catch (error) {
      console.error('Error fetching users:', error);
    } finally {
      setLoading(false);
    }
  };
  
  const handleSearch = (e: React.FormEvent) => {
    e.preventDefault();
    setPage(1);
    fetchUsers();
  };
  
  const handleSort = (sortField: string) => {
    const [currentField, currentDirection] = sort.split('_');
    const newDirection = currentField === sortField && currentDirection === 'asc' ? 'desc' : 'asc';
    setSort(`${sortField}_${newDirection}`);
  };
  
  const openDeleteConfirm = (user: User) => {
    setSelectedUser(user);
    setDeleteConfirmOpen(true);
  };
  
  const handleDelete = async () => {
    if (!selectedUser) return;
    
    try {
      setLoading(true);
      
      const response = await del(`/api/admin/users/${selectedUser._id}`);
      
      if (response.success) {
        setUsers(users.filter(user => user._id !== selectedUser._id));
        setTotalUsers(prev => prev - 1);
        setDeleteConfirmOpen(false);
        setSelectedUser(null);
      }
    } catch (error) {
      console.error('Error deleting user:', error);
    } finally {
      setLoading(false);
    }
  };
  
  const handlePageChange = (newPage: number) => {
    setPage(newPage);
    router.push({
      pathname: router.pathname,
      query: { ...router.query, page: newPage },
    }, undefined, { shallow: true });
  };
  
  const getRoleBadgeClass = (role: string) => {
    switch (role) {
      case 'admin':
        return styles.adminBadge;
      case 'superadmin':
        return styles.superAdminBadge;
      default:
        return styles.userBadge;
    }
  };
  
  const getStatusBadgeClass = (active: boolean) => {
    return active ? styles.activeBadge : styles.inactiveBadge;
  };

  return (
    <div className={styles.userTableContainer}>
      <div className={styles.tableHeader}>
        <h2>Quản lý người dùng</h2>
        <Button
          href="/admin/users/create"
          variant="primary"
          icon={<FaUserPlus />}
        >
          Thêm người dùng
        </Button>
      </div>
      
      <div className={styles.tableFilters}>
        <form onSubmit={handleSearch} className={styles.searchForm}>
          <Input
            type="text"
            placeholder="Tìm kiếm người dùng..."
            value={searchTerm}
            onChange={(e) => setSearchTerm(e.target.value)}
            rightIcon={<FaSearch />}
          />
          <Button type="submit" variant="secondary">
            Tìm kiếm
          </Button>
        </form>
        
        <div className={styles.filterControls}>
          <div className={styles.filterSelect}>
            <FaFilter />
            <select
              value={filter}
              onChange={(e) => setFilter(e.target.value)}
              aria-label="Filter users"
            >
              <option value="all">Tất cả</option>
              <option value="active">Đang hoạt động</option>
              <option value="inactive">Đã khóa</option>
              <option value="admin">Admin</option>
              <option value="user">User</option>
            </select>
          </div>
        </div>
      </div>
      
      <div className={styles.tableWrapper}>
        <table className={styles.userTable}>
          <thead>
            <tr>
              <th>
                <button
                  className={styles.sortButton}
                  onClick={() => handleSort('name')}
                  aria-label="Sort by name"
                >
                  Tên <FaSort />
                </button>
              </th>
              <th>
                <button
                  className={styles.sortButton}
                  onClick={() => handleSort('email')}
                  aria-label="Sort by email"
                >
                  Email <FaSort />
                </button>
              </th>
              <th>Vai trò</th>
              <th>
                <button
                  className={styles.sortButton}
                  onClick={() => handleSort('createdAt')}
                  aria-label="Sort by date created"
                >
                  Ngày tạo <FaSort />
                </button>
              </th>
              <th>Trạng thái</th>
              <th>Thao tác</th>
            </tr>
          </thead>
          <tbody>
            {loading && users.length === 0 ? (
              <tr>
                <td colSpan={6} className={styles.loadingCell}>
                  Đang tải...
                </td>
              </tr>
            ) : users.length > 0 ? (
              users.map((user) => (
                <tr key={user._id}>
                  <td className={styles.nameCell}>
                    <div className={styles.userAvatar}>
                      {user.avatar ? (
                        <img src={user.avatar} alt={user.name} />
                      ) : (
                        <div className={styles.avatarPlaceholder}>
                          {user.name.charAt(0)}
                        </div>
                      )}
                    </div>
                    <span>{user.name}</span>
                  </td>
                  <td>{user.email}</td>
                  <td>
                    <span className={`${styles.badge} ${getRoleBadgeClass(user.role)}`}>
                      {user.role}
                    </span>
                  </td>
                  <td>{new Date(user.createdAt).toLocaleDateString()}</td>
                  <td>
                    <span className={`${styles.badge} ${getStatusBadgeClass(user.isActive)}`}>
                      {user.isActive ? 'Hoạt động' : 'Đã khóa'}
                    </span>
                  </td>
                  <td className={styles.actionCell}>
                    <Link 
                      href={`/admin/users/edit/${user._id}`}
                      className={styles.editButton}
                      aria-label={`Edit ${user.name}`}
                    >
                      <FaEdit />
                    </Link>
                    <button
                      className={styles.deleteButton}
                      onClick={() => openDeleteConfirm(user)}
                      aria-label={`Delete ${user.name}`}
                    >
                      <FaTrash />
                    </button>
                  </td>
                </tr>
              ))
            ) : (
              <tr>
                <td colSpan={6} className={styles.emptyCell}>
                  Không tìm thấy người dùng nào
                </td>
              </tr>
            )}
          </tbody>
        </table>
      </div>
      
      <div className={styles.tablePagination}>
        <Pagination
          currentPage={page}
          totalItems={totalUsers}
          itemsPerPage={limit}
          onPageChange={handlePageChange}
        />
      </div>
      
      {/* Delete Confirmation Modal */}
      <Modal
        isOpen={deleteConfirmOpen}
        onClose={() => setDeleteConfirmOpen(false)}
        title="Xác nhận xóa người dùng"
        size="sm"
      >
        <div className={styles.deleteConfirmContent}>
          <p>
            Bạn có chắc chắn muốn xóa người dùng <strong>{selectedUser?.name}</strong>?
          </p>
          <p>Hành động này không thể hoàn tác.</p>
          
          <div className={styles.modalActions}>
            <Button
              variant="outline"
              onClick={() => setDeleteConfirmOpen(false)}
              disabled={loading}
            >
              Hủy
            </Button>
            <Button
              variant="danger"
              onClick={handleDelete}
              loading={loading}
              disabled={loading}
            >
              Xác nhận xóa
            </Button>
          </div>
        </div>
      </Modal>
    </div>
  );
};

export default UserTable;